import { Injectable } from '@nestjs/common'
import { CreateMoneyUpdateDto } from './dto/create-money-update.dto'
import { UpdateMoneyUpdateDto } from './dto/update-money-update.dto'
import { PrismaService } from 'src/prisma/prisma.service'
import { MoneyUpdate, User } from '@prisma/client'
import { QueryPaginationDto } from 'src/common/dto/pagination-query.dto'
import { UserFilterDto } from 'src/common/dto/user_filter.dto'
import { paginate } from 'src/common/utils/pagination.utils'

@Injectable()
export class MoneyUpdatesService {
  constructor(private prisma: PrismaService) { }

  async create(user: User, createMoneyUpdateDto: CreateMoneyUpdateDto): Promise<MoneyUpdate> {
    return await this.prisma.moneyUpdate.create({
      data: {
        ...createMoneyUpdateDto,
        userId: user.id,
      }
    })
  }

  async findAll(queryPagination: QueryPaginationDto, userFilter: UserFilterDto): Promise<[MoneyUpdate[], number]> {
    return [
      await this.prisma.moneyUpdate.findMany({
        where: userFilter,
        ...paginate(queryPagination),
      }),
      await this.prisma.moneyUpdate.count({
        where: userFilter,
      }),
    ]
  }

  async findOne(id: number): Promise<MoneyUpdate> {
    return await this.prisma.moneyUpdate.findUnique({
      where: { id },
    })
  }

  async update(id: number, updateMoneyUpdateDto: UpdateMoneyUpdateDto): Promise<MoneyUpdate> {
    return await this.prisma.moneyUpdate.update({
      where: { id },
      data: updateMoneyUpdateDto,
    })
  }

  async remove(id: number): Promise<MoneyUpdate> {
    return await this.prisma.moneyUpdate.delete({
      where: { id },
    })
  }
}
